#!/usr/bin/env python
# coding: utf-8

import os
import socket

import numpy as np
import cartopy.crs as ccrs
import shapely.geometry

from amrs.models.cosmo import Domain


HOST = socket.gethostname()

if HOST.startswith('daint'):
    HOST = 'daint'
elif HOST.startswith('ddm'):
    HOST = 'ddm'
else:
    HOST = None
    print('Warning: Host "%d" unknown. Data paths need to be given explicit.' % HOST )


# data path should contain the following data:
#  - cosmo_2015-01/ (currently not on ddm)
#  - cosmo_2015-07/ (currently not on ddm)
#  - cosmo2D_2015-01/
#  - cosmo2D_2015-07/ 
#  - level2/

DATA_PATH = {
    'daint': '/store/empa/em05/gkuhl/smartcarb',
    'ddm':   '/output/SMARTCARB/output'
}.get(HOST, None)


WIND_PATH = os.path.join(DATA_PATH, 'timeseries')


BG_STATS_FILENAME = { # TODO: update for daint
        'Berlin': {
            'CO2': os.path.join(DATA_PATH, 'timeseries',
                                'ts_XCO2_background_Berlin_100.dat'),
            'NO2': os.path.join(DATA_PATH, 'timeseries',
                                'ts_NO2_background_Berlin_100.dat'),
            'CO': os.path.join(DATA_PATH, 'timeseries',
                               'ts_CO_background_Berlin_100.dat'),
        },
        'Janschwalde': {
            'CO2': os.path.join(DATA_PATH, 'timeseries',
                                'ts_XCO2_background_Janschwalde_50.dat'),
            'NO2': os.path.join(DATA_PATH, 'timeseries',
                                'ts_NO2_background_Janschwalde_50.dat'),
            'CO': os.path.join(DATA_PATH, 'timeseries',
                               'ts_CO_background_Janschwalde_50.dat'),
        },
        'Lippendorf': {
            'CO2': os.path.join(DATA_PATH, 'timeseries',
                                'ts_XCO2_background_Lippendorf_50.dat'),
            'NO2': os.path.join(DATA_PATH, 'timeseries',
                                'ts_NO2_background_Lippendorf_50.dat'),
            'CO': os.path.join(DATA_PATH, 'timeseries',
                               'ts_CO_background_Lippendorf_50.dat'),
        }
}

# location of point sources in model grid (sources with >3 kt NO2 yr-1)
POINT_SOURCES = {
    'Janschwalde':    (14.4534902573, 51.841545105),
    'Boxberg':        (14.5746955872, 51.4164924622),
    'Lippendorf':     (12.3712453842, 51.1874504089),
    'Prunerov':       (13.259378168484044, 50.41673024911683),
    'Pocerady':       (13.683343307504192, 50.4247541197537),
    'Turow':          (14.9112815857, 50.9428253174),
    'Schwarze Pumpe': (14.3610925674, 51.534286499),
    'Dolna Odra':     (14.466333, 53.205776),
    'Opole':          (17.882273320866638, 50.74892801373274),
    'Schkopau':       (11.949723923708794, 51.3948184266901),
    'Patnow':         (18.2375602722, 52.3044433594),
    'Melnik':         (14.41942401738856, 50.410455288399355),
    'Heyden':         (8.988629531406984, 52.385997856504865),
    'Staudinger':     (8.957290411853883, 50.0855413919897),
    'Chvaletice':     (15.443743797083094, 50.01838901132982),
    'Belchatow':      (19.3267,            51.2659),
}

POINT_SOURCE_EMISSIONS = { # CO2 in MtCO2 yr-1, NOX in ktNO2 yr-1
    'Janschwalde':    (33.33040911098293, 26.91581803957786),
    'Boxberg':        (19.060486336342098, 15.3925893876394),
    'Lippendorf':     (15.262101429786487, 12.327907834929595),
    'Prunerov':       (11.400938940791962, 11.269198230280155),
    'Pocerady':       (9.297528526527278, 9.190058207458037),
    'Turow':          (8.719334147200808, 13.113048064532066),
    'Schwarze Pumpe': (8.214317202409294, 6.634019873974995),
    'Dolna Odra':     (7.953932364866147, 10.08281268254857),
    'Opole':          (7.4933750056659525, 9.498882653158365),
    'Schkopau':       (5.913751972021522, 4.704448672460918),
    'Patnow':         (5.828457152112036, 8.76615614258678),
    'Melnik':         (5.232576425713115, 5.172292627526416),
    'Heyden':         (5.207227529240778, 4.142311623947942),
    'Staudinger':     (4.927678855471657, 3.921285000022301),
    'Chvaletice':     (3.107421519858735, 3.0721234296899267)
}

KEY2UNICODE = {
    'Belchatow':      'Belchatow',
    'Janschwalde':    'Jänschwalde',
    'Berlin':         'Berlin',
    'Boxberg':        'Boxberg',
    'Lippendorf':     'Lippendorf',
    'Prunerov':       'Prunéřov',
    'Pocerady':       'Počerady',
    'Turow':          'Turów',
    'Schwarze Pumpe': 'Schwarze Pumpe',
    'Patnow':         'Patnów',
    'Opole':          'Opole',
    'Melnik':         'Mělník',
    'Dolna Odra':     'Dolna Odra',
    'Heyden':         'Heyden',
    'Schkopau':       'Schkopau',
    'Staudinger':     'Staudinger',
    'Chvaletice':     'Chvaletice'
}

CITIES = {
    'Berlin': (13.407696, 52.516984)
}

# used for x and y coords
EPSG = 3068
PROJ4 = (
    '+proj=cass +lat_0=52.41864827777778 +lon_0=13.62720366666667 '
    '+x_0=40000 +y_0=10000 +ellps=bessel '
    '+towgs84=612.4,77,440.2,-0.054,0.057,-2.797,2.55 +units=m +no_defs'
)
PROJ4_PARAMS = {
    'proj': 'cass',
    'lat_0': 52.41864827777778,
    'lon_0': 13.62720366666667,
    'x_0':  40000,
    'y_0':  10000,
    'ellps': 'bessel',
    'towgs84': '612.4,77,440.2,-0.054,0.057,-2.797,2.55',
    'units': 'm'
}




# settings for berlin 2 (big) domain - smartcarb production run
ie       = 700
je       = 600
ke       = 60
pollon   = -170.0  # origin lon: 10.0
pollat   =   43.0  # origin lat: 47.0
dlon     = 0.01
dlat     = 0.01
startlat =  2.5
startlon = -1.4

stoplat = startlat + (je-1) * dlat
stoplon = startlon + (ie-1) * dlon

DOMAIN = shapely.geometry.Polygon([
    (startlon - 0.5 * dlon, startlat - 0.5 * dlat),
    (stoplon  + 0.5 * dlon, startlat - 0.5 * dlat),
    (stoplon  + 0.5 * dlon, stoplat  + 0.5 * dlat),
    (startlon - 0.5 * dlon, stoplat  + 0.5 * dlat)
])


INPUT_ORG_FNAME = os.path.join(os.path.dirname(__file__), 'INPUT_ORG')
DOMAINS = {
    'smartcarb': Domain.from_nml(INPUT_ORG_FNAME),
    'europe':    Domain('europe', startlon=-12, stoplon=42,
                                  startlat=33, stoplat=72),
    'globe':     Domain('globe', startlon=-180, stoplon=180,
                                 startlat=-80, stoplat=80),
    'berlin':    Domain('berlin', startlon=1.8, stoplon=2.4,
                                  startlat=5.35, stoplat=5.75,
                                  pollon=pollon, pollat=pollat),
}

# center coords
rlon = np.linspace(startlon, stoplon, ie)
rlat = np.linspace(startlat, stoplat, je)

rlon2, rlat2 = np.meshgrid(rlon, rlat)

# corner coords
rlonc = np.linspace(startlon - 0.5 * dlon, stoplon + 0.5 * dlon, ie+1)
rlatc = np.linspace(startlat - 0.5 * dlat, stoplat + 0.5 * dlat, je+1)



WGS84 = ccrs.PlateCarree()
ROTPOLE = ccrs.RotatedPole(pole_latitude=pollat, pole_longitude=pollon)

tracer2long_name = {
    'XCO2_A': 'XCO2_from_time-varying_emissions_for_all_sources_in_domain_excl_Berli',
    'XCO2_B0': 'XCO2_from_time-varying_emissions_for_industry_in_Berlin',
    'XCO2_B1': 'XCO2_from_time-varying_emissions_for_heating_in_Berlin',
    'XCO2_B2': 'XCO2_from_time-varying_emissions_for_traffic_in_Berlin',
    'XCO2_BC': 'XCO2_from_time-constant_emissions_for_all_sources_in_Berlin',
    'XCO2_BG': 'time-varying_XCO2_at_lateral_boundary',
    'XCO2_BV': 'XCO2_from_time-varying_emissions_for_all_sources_in_Berlin',
    'XCO2_GPP': 'XCO2_from_time-varying_surface_flux_from_gross_photosynthetic_product',
    'XCO2_JC': 'XCO2_from_time-constant_emissions_for_Jaenschwalde',
    'XCO2_JV': 'XCO2_from_time-varying_emissions_for_Jaenschwalde',
    'XCO2_NPR': 'XCO2_from_time-varying_emissions_without_plume_rise',
    'XCO2_RA': 'XCO2_from_time-varying_surface_flux_from_respiration',
    'XCO2_SURF': 'XCO2_from_time-varying_emissions_for_all_sources_in_domain_but_emitte',
    'XCO2_T00': 'XCO2_from_time-varying_emissions_in_Berlin_between_00-03_UTC',
    'XCO2_T01': 'XCO2_from_time-varying_emissions_in_Berlin_between_03-06_UTC',
    'XCO2_T02': 'XCO2_from_time-varying_emissions_in_Berlin_between_06-09_UTC',
    'XCO2_T03': 'XCO2_from_time-varying_emissions_in_Berlin_between_09-12_UTC',
    'XCO2_T04': 'XCO2_from_time-varying_emissions_in_Berlin_between_12-15_UTC',
    'XCO2_T05': 'XCO2_from_time-varying_emissions_in_Berlin_between_15-18_UTC',
    'XCO2_T06': 'XCO2_from_time-varying_emissions_in_Berlin_between_18-21_UTC',
    'XCO2_T07': 'XCO2_from_time-varying_emissions_in_Berlin_between_21-24_UTC',
    'XCO2_TOT': 'XCO2_from_time-varying_emissions_for_all_sources_in_domain',
    'CO_A_column': 'CO_from_time-varying_emissions_for_all_sources_in_domain_excl_Berlin',
    'CO_B0_column': 'CO_from_time-varying_emissions_for_industry_in_Berlin',
    'CO_B1_column': 'CO_from_time-varying_emissions_for_heating_in_Berlin',
    'CO_B2_column': 'CO_from_time-varying_emissions_for_traffic_in_Berlin',
    'CO_BC_column': 'CO_from_time-constant_emissions_for_all_sources_in_Berlin',
    'CO_BG_column': 'time-varying_CO_at_lateral_boundary',
    'CO_BV_column': 'CO_from_time-varying_emissions_for_all_sources_in_Berlin',
    'CO_JC_column': 'CO_from_time-constant_emissions_for_Jaenschwalde',
    'CO_JV_column': 'CO_from_time-varying_emissions_for_Jaenschwalde',
    'CO_NPR_column': 'CO_from_time-varying_emissions_without_plume_rise',
    'CO_SURF_column': 'CO_from_time-varying_emissions_for_all_sources_in_domain_but_emitted',
    'CO_TOT_column': 'CO_from_time-varying_emissions_for_all_sources_in_domain',
    'NO2_A_column': 'NO2_from_time-varying_emissions_for_all_sources_in_domain_excl_Berli',
    'NO2_B0_column': 'NO2_from_time-varying_emissions_for_industry_in_Berlin',
    'NO2_B1_column': 'NO2_from_time-varying_emissions_for_heating_in_Berlin',
    'NO2_B2_column': 'NO2_from_time-varying_emissions_for_traffic_in_Berlin',
    'NO2_BC_column': 'NO2_from_time-constant_emissions_for_all_sources_in_Berlin',
    'NO2_BG_column': 'time-varying_NO2_at_lateral_boundary',
    'NO2_BV_column': 'NO2_from_time-varying_emissions_for_all_sources_in_Berlin',
    'NO2_JC_column': 'NO2_from_time-constant_emissions_for_Jaenschwalde',
    'NO2_JV_column': 'NO2_from_time-varying_emissions_for_Jaenschwalde',
    'NO2_NPR_column': 'NO2_from_time-varying_emissions_for_Jaenschwalde_without_plume_rise',
    'NO2_R00_column': 'NO2_from_time-varying_emissions_for_all_sources_without_decay',
    'NO2_R02_column': 'NO2_from_time-varying_emissions_for_all_sources_with_decay_time_from_2',
    'NO2_R12_column': 'NO2_from_time-varying_emissions_for_all_sources_with_decay_time_from_1',
    'NO2_R24_column': 'NO2_from_time-varying_emissions_for_all_sources_with_decay_time_from_2',
    'NO2_SURF_column': 'NO2_from_time-varying_emissions_for_all_sources_in_domain_but_emitte',
    'NO2_TOT_column': 'NO2_from_time-varying_emissions_for_all_sources_in_domain',
    'CLCT': 'total cloud cover as cloud area fraction'
}
